@@ -9,6 +9,7 @@ from starlette.applications import Starlette
 from starlette.middleware import Middleware
 from starlette.middleware.base import BaseHTTPMiddleware
 from starlette.middleware.trustedhost import TrustedHostMiddleware
+from starlette.middleware.authentication import AuthenticationMiddleware
 
 from .handlers import (
     ThingHandler,
@@ -45,7 +46,7 @@ class DefaultHeaderMiddleware(BaseHTTPMiddleware):
 
 middlewares = [
     Middleware(DefaultHeaderMiddleware),
-    Middleware(TrustedHostMiddleware, allowed_hosts=["localhost"]),
+    Middleware(TrustedHostMiddleware, allowed_hosts=["*"]),
 ]
 
 
@@ -58,8 +59,9 @@ class WebThingServer(AsyncMixin):
             things_maker,
             port=8000,
             hostname=None,
-            additional_routes=None,
             base_path="",
+            additional_routes=None,
+            additional_middlewares=None,
             additional_on_startup=None,
             additional_on_shutdown=None,
     ):
@@ -72,8 +74,9 @@ class WebThingServer(AsyncMixin):
                   SingleThing or MultipleThings
         port -- port to listen on (defaults to 80)
         hostname -- Optional host name, i.e. mything.com
-        additional_routes -- list of additional routes to add to the server
         base_path -- base URL path to use, rather than '/'
+        additional_routes -- list of additional routes to add to the server
+        additional_middlewares -- list of additional middlewares
         additional_on_startup -- list of additional starup event handlers
         additional_on_shutdown -- list of additional shutdown event handlers
         """
@@ -81,8 +84,9 @@ class WebThingServer(AsyncMixin):
         self.things = self._run_async(things_maker())
         self.port = port
         self.hostname = hostname
-        self.additional_routes = additional_routes
         self.base_path = base_path.rstrip("/") if base_path else "/things"
+        self.additional_routes = additional_routes
+        self.additional_middlewares = additional_middlewares
         self.additional_on_startup = additional_on_startup
         self.additional_on_shutdown = additional_on_shutdown
         system_hostname = socket.gethostname().lower()
@@ -172,6 +176,11 @@ class WebThingServer(AsyncMixin):
                 Mount(f"{self.base_path}", routes=routes),
             ]
 
+        middleware_lst = [] # middlewares
+        if self.additional_middlewares:
+            assert isinstance(self.additional_middlewares, list)
+            middleware_lst.extend(self.additional_middlewares)
+
         on_startups = [self.start]
         if self.additional_on_startup:
             assert isinstance(self.additional_on_startup, list)
@@ -183,11 +192,18 @@ class WebThingServer(AsyncMixin):
             on_shutdowns.extend(self.additional_on_shutdown)
 
         app = Starlette(
-            debug=True, routes=routes, on_startup=on_startups, on_shutdown=on_shutdowns,
+            debug=True, routes=routes, middleware=middleware_lst, on_startup=on_startups, on_shutdown=on_shutdowns,
         )
 
         app.state.things = self.things
 
+        require_auth = False
+        for middleware in middleware_lst:
+            if middleware.cls == AuthenticationMiddleware:
+                require_auth = True
+
+        app.state.require_auth = require_auth
+
         return app
 
     async def start(self):
